/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
var ContentLength = 'Content-Length: ';
var CRLF = '\r\n';
var StreamMessageWriter = (function () {
    function StreamMessageWriter(writable, encoding) {
        if (encoding === void 0) { encoding = 'utf8'; }
        this.writable = writable;
        this.encoding = encoding;
    }
    StreamMessageWriter.prototype.write = function (msg) {
        var json = JSON.stringify(msg);
        var contentLength = Buffer.byteLength(json, this.encoding);
        var headers = [
            ContentLength, contentLength.toString(), CRLF,
            CRLF
        ];
        // Header must be written in ASCII encoding
        this.writable.write(headers.join(''), 'ascii');
        // Now write the content. This can be written in any encoding
        this.writable.write(json, this.encoding);
    };
    return StreamMessageWriter;
})();
exports.StreamMessageWriter = StreamMessageWriter;
var IPCMessageWriter = (function () {
    function IPCMessageWriter(process) {
        this.process = process;
    }
    IPCMessageWriter.prototype.write = function (msg) {
        this.process.send(msg);
    };
    return IPCMessageWriter;
})();
exports.IPCMessageWriter = IPCMessageWriter;
